/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/addrspace.h>
#include <asm/inca-ip2.h>

#define FREQ_262MHZ 262144000
#define FREQ_327MHZ 327685000
#define FREQ_360MHZ 360448000
#define FREQ_400MHZ 393216000
#define FREQ_133MHZ (FREQ_400MHZ/3)
#define FREQ_100MHZ (FREQ_400MHZ/4)


uint incaip2_get_pllclk (void)
{
    u32 pll2_cr = (*INCA_IP2_CGU_PLL2CR & 0x018FFFFF);
    switch (pll2_cr)
    {
    case 0x01828BE3:
        return FREQ_400MHZ;
    case 0x008D8383:
        return FREQ_360MHZ;
    case 0x0085BB33:
        return FREQ_327MHZ;
    case 0x01845E93:
        return FREQ_262MHZ;
    }
    return FREQ_400MHZ;
}

uint incaip2_get_cpuclk (void)
{
	return incaip2_get_pllclk();
}

uint incaip2_get_lpbsclk (void)
{
	uint fpi_div[] = {2,3,4,5};
	uint lpbsdiv = INCA_IP2_CGU_FBSCR_LPBSDIV_GET(*INCA_IP2_CGU_FBSCR);

	return incaip2_get_pllclk()/fpi_div[lpbsdiv];
}

uint incaip2_get_fbs0clk (void)
{
	uint fbs0_div[] = {1,2,4,8};
	uint div0 = INCA_IP2_CGU_FBSCR_DIV0_GET(*INCA_IP2_CGU_FBSCR);

	return incaip2_get_lpbsclk()/fbs0_div[div0];
}

uint incaip2_get_fbs1clk (void)
{
	uint fbs1_div[] = {1,2,4,8};
	uint div1 = INCA_IP2_CGU_FBSCR_DIV1_GET(*INCA_IP2_CGU_FBSCR);

	return incaip2_get_lpbsclk()/fbs1_div[div1];
}

int incaip2_set_cpuclk (void)
{
	extern void ebu_init(long);
	extern void cgu_init(long);
	extern void sdram_init(long);
	uchar tmp[64];
	ulong cpuclk;

	if (getenv_r ("cpuclk", tmp, sizeof (tmp)) > 0) {
		cpuclk = simple_strtoul (tmp, NULL, 10) * 1000000;
		cgu_init (cpuclk);
		ebu_init (cpuclk);
		sdram_init (cpuclk);
	}

	return 0;
}
