/*
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 *  ifx_nand.c ifx_nand.h
 *
 *  Description:
 *    device driver of NAND controller of INCAIP2/Danube chip
 *  Author:
 *    Samuels Xu Liang
 *  Created:
 *    6 Sep 2005
 *  History & Modification Tag:
 * ########################################################################
 *
 *  ___________________________________________________________________________
 *  |  Tag   |                  Comments                   | Modifier & Time  |
 *  |--------+---------------------------------------------+------------------|
 *  |  S0.0  | First version of this driver and the tag is | Samuels Xu Liang |
 *  |        | implied.                                    |    6 Jul 2005    |
 *  ---------------------------------------------------------------------------
 *
 *  ___________________________________________________________________________
 *  |  Tag   |                  Comments                   | Modifier & Time  |
 *  |--------+---------------------------------------------+------------------|
 *  |  S0.1  | Second version of this driver for INCAIP2   | Winder Sung      |
 *  |        |                                             |    3 Nov 2005    |
 *  ---------------------------------------------------------------------------
 */ 
#include <common.h>

#if (CONFIG_COMMANDS & CFG_CMD_NAND)

#include "ifx_nand.h"
#include <linux/mtd/nand.h>
#include <asm/inca-ip2.h>

/*
 *  Description:
 *    Config EBU region.
 *  Input:
 *    none
 *  Output:
 *    none
 */
static inline void config_ebu(void)
{
    *IFX_EBU_ADDR_SEL = EBU_ADDR_SEL_BASE_SET(PHYSADDR(CFG_NAND_BASE) >> 12)
                         | EBU_ADDR_SEL_MASK_SET(15)
                         | EBU_ADDR_SEL_MRME_SET(0)
                         | EBU_ADDR_SEL_REGEN_SET(1);

    *IFX_EBU_CONFIG = EBU_CON_WRDIS_SET(0) | EBU_CON_ADSWP_SET(1) | EBU_CON_AGEN_SET(0x00)
                    | EBU_CON_SETUP_SET(0) | EBU_CON_WAIT_SET(0x00) | EBU_CON_WINV_SET(0)
                    | EBU_CON_PW_SET(0x00) | EBU_CON_ALEC_SET(3) | EBU_CON_BCGEN_SET(0x01)
                    | EBU_CON_WAITWRC_SET(2) | EBU_CON_WAITRDC_SET(2) | EBU_CON_HOLDC_SET(1)
                    | EBU_CON_RECOVC_SET(1) | EBU_CON_CMULT_SET(0x01);

    switch (ebu_address_region) {
        case 2:
            *INCA_IP2_GPIO_ALTSEL1 |= 0x00003000;
            *INCA_IP2_GPIO_DIR |= 0x00400000;
            break;
        case 3:
            *INCA_IP2_GPIO_ALTSEL1 |= 0x00000C00;
            *INCA_IP2_GPIO_DIR |= 0x00200000;
            break;
    }
}

/*
 *  Description:
 *    Config NAND controller.
 *  Input:
 *    none
 *  Output:
 *    none
 */
static inline void config_nand(void)
{
    /*
     *  ECC disabled
     *  CLE is pulse, all other signal are latches based
     *  CLE and ALE are active high, PRE, WP, SE and CS/CE are active low
     *  OUT_CS_S is enabled
     *  NAND mode is enabled
     */
    *IFX_NAND_CONTROL = NAND_CON_ECC_ON_SET(0)
                     | NAND_CON_PRE_LAT_EN_SET(1) | NAND_CON_WP_LAT_EN_SET(1)
                     | NAND_CON_SE_LAT_EN_SET(1) | NAND_CON_CS_LAT_EN_SET(1)
                     | NAND_CON_CLE_LAT_EN_SET(0) | NAND_CON_ALE_LAT_EN_SET(1)
                     | NAND_CON_OUT_CS_S_SET(ebu_address_region)
                     | NAND_CON_IN_CS_S_SET(ebu_address_region)
                     | NAND_CON_PRE_P_SET(1) | NAND_CON_WP_P_SET(1)
                     | NAND_CON_SE_P_SET(1) | NAND_CON_CS_P_SET(1)
                     | NAND_CON_CLE_P_SET(0) | NAND_CON_ALE_P_SET(0)
                     | NAND_CON_CSMUX_E_SET(1)
                     | NAND_CON_NANDM_SET(1);
}


/*
 * ####################################
 *           Global Function
 * ####################################
 */


/*
 * ####################################
 *           Init/Cleanup API
 * ####################################
 */

extern struct nand_chip nand_dev_desc[];

unsigned long nand_init(void)
{
    /*
     *  Configure hardware
     */
    config_ebu();
    config_nand();

    return nand_probe(CFG_NAND_BASE);
}

#endif //(CONFIG_COMMANDS & CFG_CMD_NAND)

