#include <asm/addrspace.h>
#include <asm/inca-ip2.h>

/* Bit Operation */
#define GET_BITS(x, msb, lsb)	 (((x) & ((1 << (msb + 1)) - 1)) >> (lsb))
#define SET_BITS(x, msb, lsb, value)	(((x) & ~(((1 << (msb + 1)) - 1) ^ ((1 << (lsb)) - 1))) | (((value) & ((1 << (1 + msb - lsb)) - 1)) << (lsb)))

/* Define the EBU registers */

#define IFX_NAND_EBU	INCA_IP2_EBU
#define IFX_EBU_REG	INCA_IP2_EBU

/* static volatile u8 *ebu_base_address = (volatile u8*)(KSEG1 | IFX_EBU_ADDR_REGION_BASE); */
#if (IFX_EBU_ADDR_REGION_BASE == IFX_EBU_ADDR_REGION0)
#define ebu_address_region          0
#elif (IFX_EBU_ADDR_REGION_BASE == IFX_EBU_ADDR_REGION1)
#define ebu_address_region          1
#elif (IFX_EBU_ADDR_REGION_BASE == IFX_EBU_ADDR_REGION2)
#define ebu_address_region          2
#elif (IFX_EBU_ADDR_REGION_BASE == IFX_EBU_ADDR_REGION3)
#define ebu_address_region          3
#endif


#define IFX_EBU_ADDR_SEL             ((volatile u32*)(IFX_EBU_REG + 0x20 + (ebu_address_region * 4)))
#define IFX_EBU_CONFIG                  ((volatile u32*)(IFX_EBU_REG + 0x60 + (ebu_address_region * 4)))


/* EBU Address Select Register  */
#define EBU_ADDR_SEL_BASE_SET(value)    (((value) & ((1 << (1 + 31 - 12)) - 1)) << 12)
#define EBU_ADDR_SEL_MASK_SET(value)    SET_BITS(0, 7, 4, (value))
#define EBU_ADDR_SEL_MRME_SET(value)    ((value) ? (1 << 1) : 0)
#define EBU_ADDR_SEL_REGEN_SET(value)   ((value) ? (1 << 0) : 0)

/* EBU Configuration Register */
#define EBU_CON_WRDIS_SET(value)        ((value) ? (1 << 31) : 0)
#define EBU_CON_ADSWP_SET(value)        ((value) ? (1 << 30) : 0)
#define EBU_CON_AGEN_SET(value)         SET_BITS(0, 26, 24, (value))
#define EBU_CON_SETUP_SET(value)        ((value) ? (1 << 22) : 0)
#define EBU_CON_WAIT_SET(value)         SET_BITS(0, 21, 20, (value))
#define EBU_CON_WINV_SET(value)         ((value) ? (1 << 19) : 0)
#define EBU_CON_PW_SET(value)           SET_BITS(0, 17, 16, (value))
#define EBU_CON_ALEC_SET(value)         SET_BITS(0, 15, 14, (value))
#define EBU_CON_BCGEN_SET(value)        SET_BITS(0, 13, 12, (value))
#define EBU_CON_WAITWRC_SET(value)      SET_BITS(0, 10, 8, (value))
#define EBU_CON_WAITRDC_SET(value)      SET_BITS(0, 7, 6, (value))
#define EBU_CON_HOLDC_SET(value)        SET_BITS(0, 5, 4, (value))
#define EBU_CON_RECOVC_SET(value)       SET_BITS(0, 3, 2, (value))
#define EBU_CON_CMULT_SET(value)        SET_BITS(0, 1, 0, (value))

/* Define the NAND registers */

#define IFX_NAND_REG	INCA_IP2_EBU

#define IFX_NAND_CONTROL	((volatile u32*)(IFX_NAND_REG + 0x00B0))
#define IFX_NAND_WAIT		((volatile u32*)(IFX_NAND_REG + 0x00B4))
#define IFX_NAND_ECC0		((volatile u32*)(IFX_NAND_REG + 0x00B8))
#define IFX_NAND_ECC_AC		((volatile u32*)(IFX_NAND_REG + 0x00BC))

/* NAND Flash Controller control register */

#define NAND_CON_LAT_EN                 GET_BITS(*IFX_NAND_CONTROL, 23, 18)
#define NAND_CON_ECC_ON_SET(value)      ((value) ? (1 << 31) : 0)
#define NAND_CON_PRE_LAT_EN_SET(value)  ((value) ? (1 << 23) : 0)
#define NAND_CON_WP_LAT_EN_SET(value)   ((value) ? (1 << 22) : 0)
#define NAND_CON_SE_LAT_EN_SET(value)   ((value) ? (1 << 21) : 0)
#define NAND_CON_CS_LAT_EN_SET(value)   ((value) ? (1 << 20) : 0)
#define NAND_CON_CLE_LAT_EN_SET(value)  ((value) ? (1 << 19) : 0)
#define NAND_CON_ALE_LAT_EN_SET(value)  ((value) ? (1 << 18) : 0)
#define NAND_CON_OUT_CS_S_SET(value)    SET_BITS(0, 11, 10, (value))
#define NAND_CON_IN_CS_S_SET(value)     SET_BITS(0, 9, 8, (value))
#define NAND_CON_PRE_P_SET(value)       ((value) ? (1 << 7) : 0)
#define NAND_CON_WP_P_SET(value)        ((value) ? (1 << 6) : 0)
#define NAND_CON_SE_P_SET(value)        ((value) ? (1 << 5) : 0)
#define NAND_CON_CS_P_SET(value)        ((value) ? (1 << 4) : 0)
#define NAND_CON_CLE_P_SET(value)       ((value) ? (1 << 3) : 0)
#define NAND_CON_ALE_P_SET(value)       ((value) ? (1 << 2) : 0)
#define NAND_CON_CSMUX_E_SET(value)     ((value) ? (1 << 1) : 0)
#define NAND_CON_NANDM_SET(value)       ((value) ? (1 << 0) : 0)


/*  NAND Flash Device RD/BY State Register */
#define NAND_WAIT_WR_C			(*IFX_NAND_WAIT & (1 << 3))
#define NAND_WAIT_RD			((*IFX_NAND_WAIT & (1<< 0)))
#define NAND_WAIT_RYBY			((*IFX_NAND_WAIT & 0x7) == 0x7)
#define NAND_WAIT_CLEAR_ALL()		do{*IFX_NAND_WAIT = 0x0;}while(0)

/* NAND Flash ECC Register */
#define NAND_ECC0_B2                    GET_BITS(*IFX_NAND_ECC0, 23, 16)
#define NAND_ECC0_B1                    GET_BITS(*IFX_NAND_ECC0, 15, 8)
#define NAND_ECC0_B0                    GET_BITS(*IFX_NAND_ECC0, 7, 0)

#define NAND_ECC0_CLEAR()               do { *IFX_NAND_ECC0 = 0; *IFX_NAND_ECC_AC = 0; } while (0)


/* NAND IO Port Mapping */
#define NAND_CMD_DECODER_ALE(x)         ((x) ? (1 << 2) : 0)
#define NAND_CMD_DECODER_CLE(x)         ((x) ? (1 << 3) : 0)
#define NAND_CMD_DECODER_CS(x)          ((x) ? (1 << 4) : 0)
#define NAND_CMD_DECODER_SE(x)          ((x) ? (1 << 5) : 0)
#define NAND_CMD_DECODER_WP(x)          ((x) ? (1 << 6) : 0)
#define NAND_CMD_DECODER_PRE(x)         ((x) ? (1 << 7) : 0)


#define NAND_CLE_BIT	NAND_CMD_DECODER_CLE(1);	
#define NAND_ALE_BIT	NAND_CMD_DECODER_ALE(1);
