/*****************************************************************************
 * INCA-IP2 BootROM
 * Copyright (c) 2005, Infineon Technologies AG, All rights reserved
 * COM NRW CSE VoIP
 *****************************************************************************/

/* maximum SSC FIFO size */
#define SSC_MAX_FIFO_SIZE 32

#define SSC_MODULE 0
#if (SSC_MODULE == 0) 
/* FixMe: SPI0 not avaiable in emulation database... */
/* register map of SPI0 */
#define SSC_BASE		(0xB4100400)
#else
/* register map of SPI1 */
#define SSC_BASE		(0xB4100800)
#endif

/* address of the Clock Control Register of the SSC */
#define SSC_CLC                 ((volatile u32*)(SSC_BASE+0x00))

/* address of Port Input Select Register of the SSC */
#define SSC_PISEL 		((volatile u32*)(SSC_BASE+0x04))

/* address of Identification Register of the SSC */
#define SSC_ID 			((volatile u32*)(SSC_BASE+0x08))

/* address of the Control Register of the SSC */
#define SSC_CON            	((volatile u32*)(SSC_BASE+0x10))

/* address of the Status Register of the SSC */
#define SSC_STATE          	((volatile u32*)(SSC_BASE+0x14))

/* address of the Write Hardware Modified Control Register Bits of the SSC */
#define SSC_WHBSTATE            ((volatile u32*)(SSC_BASE+0x18))

/* address of the Baudrate Timer Reload Register of the SSC */
#define SSC_BR              	((volatile u32*)(SSC_BASE+0x40))

/* address of the Baudrate Timer Status Register of the SSC */
#define SSC_BRSTAT              ((volatile u32*)(SSC_BASE+0x44))

/* address of the Transmitter Buffer Register of the SSC */
#define SSC_TB              	((volatile u32*)(SSC_BASE+0x20))

/* address of the Reciver Buffer Register of the SSC */
#define SSC_RB              	((volatile u32*)(SSC_BASE+0x24))

/* address of the Receive FIFO Control Register of the SSC */
#define SSC_RXFCON              ((volatile u32*)(SSC_BASE+0x30))

/* address of the Transmit FIFO Control Register of the SSC */
#define SSC_TXFCON              ((volatile u32*)(SSC_BASE+0x34))

/* address of the FIFO Status Register of the SSC */
#define SSC_FSTAT               ((volatile u32*)(SSC_BASE+0x38))

/* address of the Data Frame Control register of the SSC */
#define SSC_SFCON               ((volatile u32*)(SSC_BASE+0x60))

/* address of the Data Frame Status register of the SSC */
#define SSC_SFSTAT            	((volatile u32*)(SSC_BASE+0x64))

/* address of the General Purpose Output Control register of the SSC */
#define SSC_GPOCON             	((volatile u32*)(SSC_BASE+0x70))

/* address of the General Purpose Output Status register of the SSC */
#define SSC_GPOSTAT            	((volatile u32*)(SSC_BASE+0x74))

/* address of the Force GPO Status register of the SSC */
#define SSC_WHBGPOSTAT        	((volatile u32*)(SSC_BASE+0x78))

/* address of the Receive Request Register of the SSC */
#define SSC_RXREQ               ((volatile u32*)(SSC_BASE+0x80))

/* address of the Receive Count Register of the SSC */
#define SSC_RXCNT               ((volatile u32*)(SSC_BASE+0x84))

/* address of the DMA Configuration Register of the SSC */
#define SSC_DMACON            	((volatile u32*)(SSC_BASE+0xEC))

//------------------------------------------------------
// interrupt register for enabling interrupts, mask register of irq_reg
#define SSC_IRN_EN 		((volatile u32*)(SSC_BASE+0xF4))

// interrupt register for accessing interrupts
#define SSC_IRN_CR             	((volatile u32*)(SSC_BASE+0xF8))

// interrupt register for stimulating interrupts
#define SSC_IRN_ICR            	((volatile u32*)(SSC_BASE+0xFC))


/* commands for EEPROM, x25160 */
#define EEPROM_WREN			((u8)0x06)
#define EEPROM_WRDI			((u8)0x04)
#define EEPROM_RDSR			((u8)0x05)
#define EEPROM_WRSR			((u8)0x01)
#define EEPROM_READ			((u8)0x03)
#define EEPROM_WRITE			((u8)0x02)


/* bit masks for SSC registers */

/* ID register */
#define SSC_PERID_REV_MASK      0x0000001F
#define SSC_PERID_CFG_MASK      0x00000020
#define SSC_PERID_ID_MASK       0x0000FF00
#define SSC_PERID_REV_OFFSET    0
#define SSC_PERID_CFG_OFFSET    5
#define SSC_PERID_ID_OFFSET     8
#define SSC_PERID_ID            0x45
#define SSC_PERID_DMA_ON        0x00000020
#define SSC_PERID_RXFS_MASK     0x003F0000
#define SSC_PERID_RXFS_OFFSET   16
#define SSC_PERID_TXFS_MASK     0x3F000000
#define SSC_PERID_TXFS_OFFSET   24

/* PISEL register */
#define SSC_PISEL_MASTER_IN_A       0x0000
#define SSC_PISEL_MASTER_IN_B       0x0001
#define SSC_PISEL_SLAVE_IN_A        0x0000
#define SSC_PISEL_SLAVE_IN_B        0x0002
#define SSC_PISEL_CLOCK_IN_A        0x0000
#define SSC_PISEL_CLOCK_IN_B        0x0004


/* SSC_CON register */
#define SSC_CON_ECHO_MODE_ON       0x01000000
#define SSC_CON_ECHO_MODE_OFF      0x00000000
#define SSC_CON_IDLE_HIGH          0x00800000
#define SSC_CON_IDLE_LOW           0x00000000
#define SSC_CON_ENABLE_BYTE_VALID  0x00400000
#define SSC_CON_DISABLE_BYTE_VALID 0x00000000
#define SSC_CON_DATA_WIDTH_OFFSET  16
#define SSC_CON_DATA_WIDTH_MASK    0x001F0000
#define SSC_ENCODE_DATA_WIDTH(width) (((width - 1) << SSC_CON_DATA_WIDTH_OFFSET) & SSC_CON_DATA_WIDTH_MASK)

#define SSC_CON_RESET_ON_BAUDERR   0x00002000
#define SSC_CON_GO_ON_ON_BAUDERR   0x00000000

#define SSC_CON_RX_UFL_CHECK       0x00001000
#define SSC_CON_RX_UFL_IGNORE      0x00000000
#define SSC_CON_TX_UFL_CHECK       0x00000800
#define SSC_CON_TX_UFL_IGNORE      0x00000000
#define SSC_CON_ABORT_ERR_CHECK    0x00000400
#define SSC_CON_ABORT_ERR_IGNORE   0x00000000
#define SSC_CON_RX_OFL_CHECK       0x00000200
#define SSC_CON_RX_OFL_IGNORE      0x00000000
#define SSC_CON_TX_OFL_CHECK       0x00000100
#define SSC_CON_TX_OFL_IGNORE      0x00000000
#define SSC_CON_ALL_ERR_CHECK      0x00001F00
#define SSC_CON_ALL_ERR_IGNORE     0x00000000

#define SSC_CON_LOOPBACK_MODE      0x00000080
#define SSC_CON_NO_LOOPBACK        0x00000000
#define SSC_CON_HALF_DUPLEX        0x00000080
#define SSC_CON_FULL_DUPLEX        0x00000000

#define SSC_CON_CLOCK_FALL         0x00000040
#define SSC_CON_CLOCK_RISE         0x00000000
#define SSC_CON_SHIFT_THEN_LATCH   0x00000000
#define SSC_CON_LATCH_THEN_SHIFT   0x00000020
#define SSC_CON_MSB_FIRST          0x00000010
#define SSC_CON_LSB_FIRST          0x00000000
#define SSC_CON_ENABLE_CSB         0x00000008
#define SSC_CON_DISABLE_CSB        0x00000000
#define SSC_CON_INVERT_CSB         0x00000004
#define SSC_CON_TRUE_CSB           0x00000000
#define SSC_CON_RX_OFF             0x00000002
#define SSC_CON_RX_ON              0x00000000
#define SSC_CON_TX_OFF             0x00000001
#define SSC_CON_TX_ON              0x00000000


/* SSC_STATE register */
#define SSC_STATE_RX_BYTE_VALID_OFFSET 28
#define SSC_STATE_RX_BYTE_VALID_MASK   0x70000000
#define SSC_DECODE_RX_BYTE_VALID(con_state) ((con_state & SSC_STATE_RX_BYTE_VALID_MASK) >> SSC_STATE_RX_BYTE_VALID_OFFSET)
#define SSC_STATE_TX_BYTE_VALID_OFFSET 24
#define SSC_STATE_TX_BYTE_VALID_MASK   0x07000000
#define SSC_DECODE_TX_BYTE_VALID(con_state) ((con_state & SSC_STATE_TX_BYTE_VALID_MASK) >> SSC_STATE_TX_BYTE_VALID_OFFSET)
#define SSC_STATE_BIT_COUNT_OFFSET     16
#define SSC_STATE_BIT_COUNT_MASK       0x001F0000
#define SSC_DECODE_DATA_WIDTH(con_state) (((con_state & SSC_STATE_BIT_COUNT_MASK) >> SSC_STATE_BIT_COUNT_OFFSET) + 1)
#define SSC_STATE_BUSY                 0x00002000
#define SSC_STATE_RX_UFL               0x00001000
#define SSC_STATE_TX_UFL               0x00000800
#define SSC_STATE_ABORT_ERR            0x00000400
#define SSC_STATE_RX_OFL               0x00000200
#define SSC_STATE_TX_OFL               0x00000100
#define SSC_STATE_MODE_ERR             0x00000080
#define SSC_STATE_SLAVE_IS_SELECTED    0x00000004
#define SSC_STATE_IS_MASTER            0x00000002
#define SSC_STATE_IS_ENABLED           0x00000001
			      
/* WHBSTATE register */
#define SSC_WHBSTATE_DISABLE_SSC        0x0001
#define SSC_WHBSTATE_CONFIGURATION_MODE 0x0001
#define SSC_WHBSTATE_CLR_ENABLE         0x0001

#define SSC_WHBSTATE_ENABLE_SSC         0x0002
#define SSC_WHBSTATE_RUN_MODE           0x0002
#define SSC_WHBSTATE_SET_ENABLE         0x0002

#define SSC_WHBSTATE_SLAVE_MODE         0x0004
#define SSC_WHBSTATE_CLR_MASTER_SELECT  0x0004

#define SSC_WHBSTATE_MASTER_MODE        0x0008
#define SSC_WHBSTATE_SET_MASTER_SELECT  0x0008

#define SSC_WHBSTATE_CLR_RX_UFL_ERROR   0x0010
#define SSC_WHBSTATE_SET_RX_UFL_ERROR   0x0020

#define SSC_WHBSTATE_CLR_MODE_ERROR     0x0040
#define SSC_WHBSTATE_SET_MODE_ERROR     0x0080

#define SSC_WHBSTATE_CLR_TX_OFL_ERROR   0x0100
#define SSC_WHBSTATE_CLR_RX_OFL_ERROR   0x0200
#define SSC_WHBSTATE_CLR_ABORT_ERROR    0x0400
#define SSC_WHBSTATE_CLR_TX_UFL_ERROR   0x0800
#define SSC_WHBSTATE_SET_TX_OFL_ERROR   0x1000
#define SSC_WHBSTATE_SET_RX_OFL_ERROR   0x2000
#define SSC_WHBSTATE_SET_ABORT_ERROR    0x4000
#define SSC_WHBSTATE_SET_TX_UFL_ERROR   0x8000
#define SSC_WHBSTATE_CLR_ALL_ERROR      0x0F50
#define SSC_WHBSTATE_SET_ALL_ERROR      0xF0A0

/* BR register */
#define SSC_BR_BAUDRATE_OFFSET      0
#define SSC_BR_BAUDRATE_MASK        0xFFFF

/* BR_STAT register */
#define SSC_BRSTAT_BAUDTIMER_OFFSET      0
#define SSC_BRSTAT_BAUDTIMER_MASK        0xFFFF

/* TB register */
#define SSC_TB_DATA_OFFSET      0
#define SSC_TB_DATA_MASK        0xFFFFFFFF

/* RB register */
#define SSC_RB_DATA_OFFSET      0
#define SSC_RB_DATA_MASK        0xFFFFFFFF


/* RXFCON and TXFCON registers */
#define SSC_XFCON_FIFO_DISABLE      0x0000
#define SSC_XFCON_FIFO_ENABLE       0x0001
#define SSC_XFCON_FIFO_FLUSH        0x0002
#define SSC_XFCON_ITL_MASK          0x00003F00
#define SSC_XFCON_ITL_OFFSET        8

/* FSTAT register */
#define SSC_FSTAT_RECEIVED_WORDS_OFFSET  0
#define SSC_FSTAT_RECEIVED_WORDS_MASK    0x003F
#define SSC_FSTAT_TRANSMIT_WORDS_OFFSET  8
#define SSC_FSTAT_TRANSMIT_WORDS_MASK    0x3F00

/* GPOCON register */
#define SSC_GPOCON_INVOUT0_POS      0
#define SSC_GPOCON_INV_OUT0         0x00000001
#define SSC_GPOCON_TRUE_OUT0        0x00000000
#define SSC_GPOCON_INVOUT1_POS      1
#define SSC_GPOCON_INV_OUT1         0x00000002
#define SSC_GPOCON_TRUE_OUT1        0x00000000
#define SSC_GPOCON_INVOUT2_POS      2
#define SSC_GPOCON_INV_OUT2         0x00000003
#define SSC_GPOCON_TRUE_OUT2        0x00000000
#define SSC_GPOCON_INVOUT3_POS      3
#define SSC_GPOCON_INV_OUT3         0x00000008
#define SSC_GPOCON_TRUE_OUT3        0x00000000
#define SSC_GPOCON_INVOUT4_POS      4
#define SSC_GPOCON_INV_OUT4         0x00000010
#define SSC_GPOCON_TRUE_OUT4        0x00000000
#define SSC_GPOCON_INVOUT5_POS      5
#define SSC_GPOCON_INV_OUT5         0x00000020
#define SSC_GPOCON_TRUE_OUT5        0x00000000
#define SSC_GPOCON_INVOUT6_POS      6
#define SSC_GPOCON_INV_OUT6         0x00000040
#define SSC_GPOCON_TRUE_OUT6        0x00000000
#define SSC_GPOCON_INVOUT7_POS      7
#define SSC_GPOCON_INV_OUT7         0x00000080
#define SSC_GPOCON_TRUE_OUT7        0x00000000
#define SSC_GPOCON_INV_OUT_ALL      0x000000FF
#define SSC_GPOCON_TRUE_OUT_ALL     0x00000000

#define SSC_GPOCON_ISCSB0_POS       8
#define SSC_GPOCON_IS_CSB0          0x00000100
#define SSC_GPOCON_IS_GPO0          0x00000000
#define SSC_GPOCON_ISCSB1_POS       9
#define SSC_GPOCON_IS_CSB1          0x00000200
#define SSC_GPOCON_IS_GPO1          0x00000000
#define SSC_GPOCON_ISCSB2_POS       10
#define SSC_GPOCON_IS_CSB2          0x00000400
#define SSC_GPOCON_IS_GPO2          0x00000000
#define SSC_GPOCON_ISCSB3_POS       11
#define SSC_GPOCON_IS_CSB3          0x00000800
#define SSC_GPOCON_IS_GPO3          0x00000000
#define SSC_GPOCON_ISCSB4_POS       12
#define SSC_GPOCON_IS_CSB4          0x00001000
#define SSC_GPOCON_IS_GPO4          0x00000000
#define SSC_GPOCON_ISCSB5_POS       13
#define SSC_GPOCON_IS_CSB5          0x00002000
#define SSC_GPOCON_IS_GPO5          0x00000000
#define SSC_GPOCON_ISCSB6_POS       14
#define SSC_GPOCON_IS_CSB6          0x00004000
#define SSC_GPOCON_IS_GPO6          0x00000000
#define SSC_GPOCON_ISCSB7_POS       15
#define SSC_GPOCON_IS_CSB7          0x00008000
#define SSC_GPOCON_IS_GPO7          0x00000000
#define SSC_GPOCON_IS_CSB_ALL       0x0000FF00
#define SSC_GPOCON_IS_GPO_ALL       0x00000000

/* GPOSTAT register */
#define SSC_GPOSTAT_OUT0            0x00000001
#define SSC_GPOSTAT_OUT1            0x00000002
#define SSC_GPOSTAT_OUT2            0x00000004
#define SSC_GPOSTAT_OUT3            0x00000008
#define SSC_GPOSTAT_OUT4            0x00000010
#define SSC_GPOSTAT_OUT5            0x00000020
#define SSC_GPOSTAT_OUT6            0x00000040
#define SSC_GPOSTAT_OUT7            0x00000080
#define SSC_GPOSTAT_OUT_ALL         0x000000FF

/* WHBGPOSTAT register */
#define SSC_WHBGPOSTAT_CLROUT0_POS  0
#define SSC_WHBGPOSTAT_CLR_OUT0     0x00000001
#define SSC_WHBGPOSTAT_CLROUT1_POS  1
#define SSC_WHBGPOSTAT_CLR_OUT1     0x00000002
#define SSC_WHBGPOSTAT_CLROUT2_POS  2
#define SSC_WHBGPOSTAT_CLR_OUT2     0x00000004
#define SSC_WHBGPOSTAT_CLROUT3_POS  3
#define SSC_WHBGPOSTAT_CLR_OUT3     0x00000008
#define SSC_WHBGPOSTAT_CLROUT4_POS  4
#define SSC_WHBGPOSTAT_CLR_OUT4     0x00000010
#define SSC_WHBGPOSTAT_CLROUT5_POS  5
#define SSC_WHBGPOSTAT_CLR_OUT5     0x00000020
#define SSC_WHBGPOSTAT_CLROUT6_POS  6
#define SSC_WHBGPOSTAT_CLR_OUT6     0x00000040
#define SSC_WHBGPOSTAT_CLROUT7_POS  7
#define SSC_WHBGPOSTAT_CLR_OUT7     0x00000080
#define SSC_WHBGPOSTAT_CLR_OUT_ALL  0x000000FF

#define SSC_WHBGPOSTAT_SETOUT0_POS  8
#define SSC_WHBGPOSTAT_SET_OUT0     0x00000100
#define SSC_WHBGPOSTAT_SETOUT1_POS  9
#define SSC_WHBGPOSTAT_SET_OUT1     0x00000200
#define SSC_WHBGPOSTAT_SETOUT2_POS  10
#define SSC_WHBGPOSTAT_SET_OUT2     0x00000400
#define SSC_WHBGPOSTAT_SETOUT3_POS  11
#define SSC_WHBGPOSTAT_SET_OUT3     0x00000800
#define SSC_WHBGPOSTAT_SETOUT4_POS  12
#define SSC_WHBGPOSTAT_SET_OUT4     0x00001000
#define SSC_WHBGPOSTAT_SETOUT5_POS  13
#define SSC_WHBGPOSTAT_SET_OUT5     0x00002000
#define SSC_WHBGPOSTAT_SETOUT6_POS  14
#define SSC_WHBGPOSTAT_SET_OUT6     0x00004000
#define SSC_WHBGPOSTAT_SETOUT7_POS  15
#define SSC_WHBGPOSTAT_SET_OUT7     0x00008000
#define SSC_WHBGPOSTAT_SET_OUT_ALL  0x0000FF00

/* SFCON register */
#define SSC_SFCON_SF_ENABLE                0x00000001
#define SSC_SFCON_SF_DISABLE               0x00000000
#define SSC_SFCON_FIR_ENABLE_BEFORE_PAUSE  0x00000004
#define SSC_SFCON_FIR_DISABLE_BEFORE_PAUSE 0x00000000
#define SSC_SFCON_FIR_ENABLE_AFTER_PAUSE   0x00000008
#define SSC_SFCON_FIR_DISABLE_AFTER_PAUSE  0x00000000
#define SSC_SFCON_DATA_LENGTH_MASK         0x0000FFF0
#define SSC_SFCON_DATA_LENGTH_OFFSET       4
#define SSC_SFCON_PAUSE_CLOCK_0            0x00000000
#define SSC_SFCON_PAUSE_CLOCK_1            0x00010000
#define SSC_SFCON_PAUSE_CLOCK_IDLE         0x00020000
#define SSC_SFCON_PAUSE_CLOCK_RUN          0x00030000
#define SSC_SFCON_PAUSE_DATA_0             0x00000000
#define SSC_SFCON_PAUSE_DATA_1             0x00040000
#define SSC_SFCON_PAUSE_DATA_IDLE          0x00080000
#define SSC_SFCON_STOP_AFTER_PAUSE         0x00100000
#define SSC_SFCON_CONTINUE_AFTER_PAUSE     0x00000000
#define SSC_SFCON_PAUSE_LENGTH_MASK        0xFFC00000
#define SSC_SFCON_PAUSE_LENGTH_OFFSET      22

#define SSC_SFCON_EXTRACT_DATA_LENGTH(sfcon)  ((sfcon & SSC_SFCON_DATA_LENGTH_MASK) >> SSC_SFCON_DATA_LENGTH_OFFSET)
#define SSC_SFCON_EXTRACT_PAUSE_LENGTH(sfcon) ((sfcon & SSC_SFCON_PAUSE_LENGTH_MASK) >> SSC_SFCON_PAUSE_LENGTH_OFFSET)
#define SSC_SFCON_SET_DATA_LENGTH(value)      ((value << SSC_SFCON_DATA_LENGTH_OFFSET) & SSC_SFCON_DATA_LENGTH_MASK)
#define SSC_SFCON_SET_PAUSE_LENGTH(value)      ((value << SSC_SFCON_PAUSE_LENGTH_OFFSET) & SSC_SFCON_PAUSE_LENGTH_MASK)

/* SFSTAT register */
#define SSC_SFSTAT_IN_DATA            0x00000001
#define SSC_SFSTAT_IN_PAUSE           0x00000002
#define SSC_SFSTAT_DATA_COUNT_MASK    0x0000FFF0
#define SSC_SFSTAT_DATA_COUNT_OFFSET  4
#define SSC_SFSTAT_PAUSE_COUNT_MASK   0xFFF00000
#define SSC_SFSTAT_PAUSE_COUNT_OFFSET 20

#define SSC_SFSTAT_EXTRACT_DATA_COUNT(sfstat) ((sfstat & SSC_SFSTAT_DATA_COUNT_MASK) >> SSC_SFSTAT_DATA_COUNT_OFFSET)
#define SSC_SFSTAT_EXTRACT_PAUSE_COUNT(sfstat) ((sfstat & SSC_SFSTAT_PAUSE_COUNT_MASK) >> SSC_SFSTAT_PAUSE_COUNT_OFFSET)

/* RXREQ register */
#define SSC_RXREQ_RXCOUNT_MASK       0x0000FFFF
#define SSC_RXREQ_RXCOUNT_OFFSET     0

/* RXCNT register */
#define SSC_RXCNT_TODO_MASK       0x0000FFFF
#define SSC_RXCNT_TODO_OFFSET     0

/* DMACON register */
#define SSC_DMACON_RXON           0x00000001
#define SSC_DMACON_RXOFF          0x00000000
#define SSC_DMACON_TXON           0x00000002
#define SSC_DMACON_TXOFF          0x00000000
#define SSC_DMACON_DMAON          0x00000003
#define SSC_DMACON_DMAOFF         0x00000000
#define SSC_DMACON_CLASS_MASK     0x0000000C
#define SSC_DMACON_CLASS_OFFSET   2

