#ifndef __SERIAL_FLASH_H__
#define __SERIAL_FLASH_H__

#ifdef CONFIG_SERIAL_FLASH

#define STM_M25P64

/*************************************************************************************************************************/
#ifdef STM_M25P64

  /****************************************************************/
 /* M25P64-specific Declarations                                 */
/****************************************************************/

/* M25P64 Serial Flash Memory Instruction Set */
#define SERIAL_FLASH_WREN	0x06	/* Write Enable */
#define SERIAL_FLASH_WRDI	0x04	/* Write Disable */
#define SERIAL_FLASH_RDID	0x9F
#define SERIAL_FLASH_RDSR	0x05	/* Read Status Register */
#define SERIAL_FLASH_WRSR	0x01	/* Write Status Register */
#define SERIAL_FLASH_READ	0x03
#define SERIAL_FLASH_FAST_READ	0x0B
#define SERIAL_FLASH_PP		0x02	/* Page Program */
#define SERIAL_FLASH_SE		0xD8	/* Sector Erase */
#define SERIAL_FLASH_BE		0xC7	/* Bulk Erase */

/* Status Register bits */
#define SERIAL_FLASH_SR_SRWD	1 << 7
#define SERIAL_FLASH_SR_WEL	1 << 1
#define SERIAL_FLASH_SR_WIP	1 << 0
#define SERIAL_FLASH_SR_BP_MASK		7 << 2
#define SERIAL_FLASH_SR_BP(value)	(7 & value) << 2
#define SERIAL_FLASH_SR_BP_MAX		7

#define SERIAL_FLASH_PP_TIMEOUT		(5 * CFG_HZ) >> 10	/* To divide by a 1000 -> shift 10 bits to the right */
#define SERIAL_FLASH_WRSR_TIMEOUT	(15 * CFG_HZ) >> 10
#define SERIAL_FLASH_SE_TIMEOUT		(3 * CFG_HZ)
#define SERIAL_FLASH_BE_TIMEOUT		(160 * CFG_HZ)
#define SERIAL_FLASH_BE_SUB_TIMEOUT	(40 * CFG_HZ)	/* 160 is too long for the get_timer ulong return value */

 /* Macros: Serial flash operation */
/**********************************/

#define SF_get_sect_number(value)	(value & 0xFF0000) >> 16

#endif /* STM_M25P64 defined */
/*************************************************************************************************************************/

/* Device IDs */
#define M25P64	 		0x202017
#define M25P16	 		0x202015
#define SERIAL_FLASH_UNKNOWN	0xFFFFFF


  /*******************************************************/
 /* Serial Flash Structures                             */
/*******************************************************/

typedef struct {
	ulong	size;
	ushort	page_size;
	ulong	sector_size;
	ushort	sector_count;
	ulong	serial_flash_id;
	ulong	logical_address;
	ulong	sector_start[CFG_MAX_SFLASH_SECT];
	uchar	sector_protect[CFG_MAX_SFLASH_SECT];
	uchar	sector_real_protect[CFG_MAX_SFLASH_SECT];

} serial_flash_info_t;

enum {
	NOT_PROTECTED,
	PROTECTED
};


  /*****************************************************/
 /* Function Declarations                             */
/*****************************************************/

/* serial_flash.c */
extern int   serial_flash_init(void);
extern int   serial_flash_addr(ulong addr);
extern void  serial_flash_print_info(void);
extern int   serial_flash_read(ulong logic_addr, ulong size, uchar *result);
extern int   serial_flash_write(ulong logic_dest_addr, ulong size, ulong source_addr);
extern int   serial_flash_erase(ushort start_sector, ushort end_sector);
extern int   serial_flash_sect_erase(ulong logic_start_addr, ulong logic_end_addr);
extern int   serial_flash_bulk_erase(void);
extern int   serial_flash_block_protect(uchar);
extern int   serial_flash_protect(int flag, ulong logic_start_addr, ulong logic_end_addr);
extern void  serial_flash_perror(int err);

/* sf.c, functions specific to the serial flash memory chip used */
extern void SF_init(void);
extern int  SF_read_id(ulong *id);
extern int  SF_read_status_register(uchar *sr);
extern int  SF_write_status_register(uchar sr);
extern int  SF_page_program(ulong addr, ulong size, uchar *data);
extern int  SF_sector_erase(ulong addr);
extern int  SF_bulk_erase(void);
extern int  SF_read(ulong addr, ulong size, uchar *result);
extern int  SF_fast_read(ulong addr, ulong size, uchar *result);
extern int  SF_block_protect(uchar bp);

/* Return values for SF_ functions */
#define SFLASH_OK		0
#define SFLASH_TIMOUT		1
#define SFLASH_NOT_ERASED	2
#define SFLASH_PROTECTED	4
#define SFLASH_INVAL		8
#define SFLASH_UNKNOWN_FLASH	16
#define SFLASH_OVERFLOW		32
#define SFLASH_ALL_PROT		64
 
#endif

#endif /* CONFIG_SERIAL_FLASH */
